package game;

import sk.thenet.plustd.FM;

class Action {
	public var skippable:Bool = false;
	public function tick(gs:GSGame):Bool {return true;}
}

class Wait extends Action {
	public var wait:Int;
	
	public function new(nwait:Int){
		wait = nwait;
	}
	
	override public function tick(gs:GSGame):Bool {
		wait--;
		return (wait <= 0);
	}
}

class Execute extends Action {
	public var exec:GSGame->Bool;
	
	public function new(nexec:GSGame->Bool){
		exec = nexec;
	}
	
	override public function tick(gs:GSGame):Bool {
		return exec(gs);
	}
}

class LoseItem extends Action {
	public var iid:Int;
	
	public function new(niid:Int){
		iid = niid;
	}
	
	override public function tick(gs:GSGame):Bool {
		var iof = gs.inventory.indexOf(iid);
		if (iof >= 0) gs.inventory.splice(iof, 1);
		return true;
	}
}

class Dialogue extends Action {
	public var x:Int;
	public var y:Int;
	public var font:Int;
	public var txt:String;
	public var dur:Int;
	public var glitchy:Bool;
	public var glitches:Array<Array<Int>>;
	
	public static function playerSay(ntxt:String, ndur:Int = 0):Dialogue {
		var p = cast(Main.st.cstr, GSGame).player;
		var pos = p.getRealPos();
		var w:Int = 0;
		var ts:Array<String> = ntxt.split("\n");
		for (t in ts) if (w < t.length) w = t.length;
		return new Dialogue(pos.x - FM.floor(w * 3.5), pos.y - (p.small ? 65 : 105) - ts.length * 12, 0, ntxt, ndur);
	}
	
	public function new(nx:Int, ny:Int, nfont:Int, ntxt:String, ndur:Int = 0, nglitchy:Bool = false){
		skippable = true;
		x = nx;
		y = ny;
		if (x < 0) x = 0;
		var w:Int = 0;
		var ts:Array<String> = ntxt.split("\n");
		for (t in ts) if (w < t.length) w = t.length;
		if (x + w * 7 >= 320) x = 320 - w * 7;
		if (y < 0) y = 0;
		if (y + ts.length * 12 >= 190) y = 190 - ts.length * 12;
		font = nfont;
		txt = ntxt;
		dur = ndur;
		if (dur == 0){
			dur = ntxt.split(" ").length * 20;//(200 wpm = 3.3 wps = 0,11 wpf)
			dur = (dur < 35 ? 35 : dur);
		}
		glitchy = nglitchy;
		if (glitchy){
			glitches = [];
			for (i in 0...txt.length) glitches[i] = [0, 0];
		}
	}
	
	override public function tick(gs:GSGame):Bool {
		gs.dialogue_show = true;
		gs.dialogue_x = x;
		gs.dialogue_y = y;
		gs.dialogue_font = font;
		if (glitchy){
			if (Math.random() > .81) gs.dialogue_x += 2;
			var ntxt:String = txt.substr(0);
			var glch:Array<String> = ["`", "{", "|", "}", "~", String.fromCharCode(127), "#", "$", "%"];
			for (n in 0...ntxt.length){
				if (glitches[n][0] > 0){
					ntxt = ntxt.substr(0, n) + glch[glitches[n][1]] + ntxt.substr(n + 1);
					glitches[n][0]--;
				} else if (ntxt.charAt(n) != "\n" && Math.random() > .98){
					glitches[n][0] = 3 + FM.floor(Math.random() * Math.random() * 4);
					glitches[n][1] = FM.floor(Math.random() * 9);
					ntxt = ntxt.substr(0, n) + glch[glitches[n][1]] + ntxt.substr(n + 1);
				}
			}
			gs.dialogue_txt = ntxt;
		} else {
			gs.dialogue_txt = txt;
		}
		dur--;
		return dur <= 0;
	}
}