package game;

import sk.thenet.plustd.FM;
import sk.thenet.gamestd.pathfind.PFAstar;
import sk.thenet.gamestd.*;
import sk.thenet.gamestd.bmp.BitmapData;

class GSGame extends StaterState {
	// global variables
	public var screens:Array<Screen>;
	public var items:Array<Item>;
	public var npcs:Array<Actor>;
	
	// player variables
	public var inventory:Array<Int>;
	public var player:APlayer;
	
	public var action_queue:Array<Action>;
	
	public static inline var PM_FREE:Int = 0;
	public static inline var PM_SELECTING_INTERACTION:Int = 1;
	public static inline var PM_TALKING:Int = 2;
	public static inline var PM_ACTION:Int = 3;
	// ... ?
	public var player_mode:Int;
	public var player_screen:Int;
	public var player_x:Int;
	public var player_y:Int;
	
	// PM_FREE
	public var player_moi:Interactive;
	public var player_walkable:Bool;
	public var player_walkx:Int;
	public var player_walky:Int;
	public var player_pfa:PFAstar;
	public var player_pfapath:Array<Int>;
	public var player_pfatari:Interaction;
	public var player_walkphase:Float = 0;
	
	// PM_SELECTING_INTERACTION
	public var player_selection:Array<Interaction>;
	public var player_selx:Int;
	public var player_sely:Int;
	public var player_selw:Int;
	public var player_selh:Int;
	public var player_selmoi:Interaction;
	
	// PM_TALKING
	public var player_diatree:Array<Array<Interaction>>;
	public var player_diacur:Int;
	public var player_diasel:Interaction;
	
	// dialogue
	public var dialogue_show:Bool = false;
	public var dialogue_font:Int;
	public var dialogue_x:Int;
	public var dialogue_y:Int;
	public var dialogue_txt:String;
	
	// fade
	public var fadecol:UInt = 0x2B5531;
	public var fade:Int = 100;
	public var fadetar:Int = 0;
	
	// vr
	public var vr_disp:Int = 0;
	public var vr_disptar:Int = 0;
	public var vr_orig:BitmapData;
	public var vr_bd:BitmapData;
	public var vr_phase:Int = 0;
	
	public var ftick:Bool = true;
	
	public function new(){
		super("game");
		
		screens = [
			new SJungle(),
			new SFacility(),
			new SGarage()
			/*new SFence(),
			new SGuns(),
			new SBar(),
			new SGarage()*/
		];
		items = [
			new IDatajar(0),
			new IDatajar(1),
			new IVRHeadset()
			/*
			new IDatajarBottle(),
			new IDatajarPlug(),
			new IRifle(),
			new ICredits()*/
		];
		npcs = [
			/*new AMachine(),
			new ABob(),
			new AMarley(),
			new AIrma()*/
		];
		
		vr_bd = new BitmapData(320, 190, 0);
		inventory = [1];
		player_pfa = new PFAstar();
		player_screen = -1;
		switchScreen(0, 1, 2);
		
		player_mode = PM_FREE;
		action_queue = [];
		
		player_moi = null;
		
		player_selection = null;
	}
	
	public function switchScreen(nscrid:Int, npx:Int, npy:Int):Void {
		if (player_screen != -1){
			for (i in 0...screens[player_screen].interactives.length){
				if (screens[player_screen].interactives[i].isPlayer){
					screens[player_screen].interactives.splice(i, 1);
					break;
				}
			}
			screens[player_screen].uninit();
		}
		player_screen = nscrid;
		player = new APlayer(screens[player_screen]);
		if (inventory.indexOf(2) != -1) player.vr = true;
		screens[player_screen].init();
		if (screens[player_screen].playerSmall) player.small = true;
		if (screens[player_screen].playerMedium) player.medium = true;
		screens[player_screen].interactives.push(player);
		screens[player_screen].updatePFA(player_pfa);
		player_pfapath = [];
		player_x = npx;
		player_y = npy;
	}
	
	override public function fFrame(s:Stater):Void {
		if (ftick){
			player_mode = PM_ACTION;
			action_queue = [
				Action.Dialogue.playerSay(
					"Ugh... Where am I?"),
				Action.Dialogue.playerSay(
					"I don't remember going here.")
			];
			ftick = false;
		}
		
		//s.bd.fillRect(0, 0, 320, 240, 0xFF000000);
		var curscr:Screen = screens[player_screen];
		
		curscr.tick(this);
		curscr.interactives.sort(function(a:Interactive, b:Interactive):Int {
			var az:Int = a.getZ();
			var bz:Int = b.getZ();
			return (az < bz ? -1 : (az > bz ? 1 : 0));
		});
		
		if (player_pfapath.length > 0){
			player_walkphase += (player.small ? 0.025 : .05) * curscr.walkmap_speedmul;
			if (player_walkphase >= 1){
				player_walkphase -= 1;
				player_x += player_pfapath.shift();
				player_y += player_pfapath.shift();
				if (player_pfapath.length == 0 && player_pfatari != null){
					player_pfatari.exec(this);
					player_pfatari = null;
				}
			}
		}
		
		if (action_queue.length > 0){
			while (action_queue.length > 0 && action_queue[0].tick(this)) action_queue.shift();
			if (player_mode == PM_ACTION && action_queue.length == 0) player_mode = PM_FREE;
		}
		
		player_moi = null;
		player_selmoi = null;
		player_diasel = null;
		
		var bg_rendered:Bool = false;
		for (inter in curscr.interactives){
			if (inter.getZ() >= 0 && !bg_rendered){
				s.bd.copyPixels(curscr.bg, 0, 0,
					curscr.bg_x, curscr.bg_y, 320, 190);
				bg_rendered = true;
			}
			inter.render(this, s.bd, curscr.bg_x, curscr.bg_y);
			if (inter.hasint && !inter.isPlayer && s.ms.getX() >= 0 && s.ms.getX() < 320
				&& s.ms.getY() >= 0 && s.ms.getY() < 240 && inter.isMouseover(s.ms.getX(), s.ms.getY(),
				curscr.bg_x, curscr.bg_y)){
				player_moi = inter;
			}
		}
		if (curscr.hasfg)
			s.bd.copyPixels(curscr.fg, 0, 0,
				curscr.fg_x, curscr.fg_y, 320, 190);
		
		var ti:Int = 0;
		player_walkable = false;
		if (s.ms.getX() >= 0 && s.ms.getX() < 320 && s.ms.getY() >= 0 && s.ms.getY() < 190)
			for (y in 0...curscr.walkmap_h) for (x in 0...curscr.walkmap_w){
				var tx:Int = curscr.walkmap_x + FM.floor(x * curscr.walkmap_xx + y * curscr.walkmap_xy) - curscr.bg_x;
				var ty:Int = curscr.walkmap_y + FM.floor(x * curscr.walkmap_yx + y * curscr.walkmap_yy) - curscr.bg_y;
				if (s.ms.getX() >= tx - curscr.walkmap_dist && s.ms.getX() < tx + curscr.walkmap_dist
					&& s.ms.getY() >= ty - curscr.walkmap_dist - (curscr.walkmap[ti].back ? curscr.walkmap_backadd : 0)
					&& s.ms.getY() < ty + curscr.walkmap_dist && curscr.walkmap[ti].isWalkable()){
					player_walkable = true;
					player_walkx = x;
					player_walky = y;
				}
				// DEBUG: walkmap
				/*s.bd.set(tx, ty, curscr.walkmap[ti].isWalkable() ? 0xFFFF00FF : 0xFF00FF00);
				//*/
				ti++;
			}
		
		// VR
		if (vr_disp != vr_disptar) vr_disp += (vr_disp < vr_disptar ? 1 : -1);
		if (vr_disp > 0){
			if (vr_phase == 0){
				VR.repoint();
			}
			VR.virtualise(vr_bd, vr_orig, vr_phase);
			vr_phase++;
			if (vr_phase >= 19) vr_phase = 0;
			var hilbdit:Array<Int> = [
				0, 8, 7,15,
				9, 4,14, 3,
				2, 11,1,13,
				10,5,12, 6,
			];
			var vec = s.bd.getFullVector();
			var vrv = vr_bd.getFullVector();
			var i:Int = 0;
			for (y in 0...190) for (x in 0...320){
				if (hilbdit[(y % 4) * 4 + (x % 4)] < (vr_disp / 100) * 16 && (vrv[i] & 0xFF000000) != 0){
					vec[i] |= vrv[i];
				}
				i++;
			}
			s.bd.setFullVector(vec);
		}
		
		// fade, flash
		if (fade != fadetar) fade += (fade < fadetar ? 2 : -2);
		if (fade != 0){
			var fadebd = new BitmapData(320, 190, (FM.floor((fade / 100) * 255) << 24) | fadecol);
			s.bd.copyPixels(fadebd, 0, 0, 0, 0, 320, 190);
		}
		
		// dialogue
		if (dialogue_show){
			(switch (dialogue_font){
				case 1: Assets.fMachine;
				case 2: Assets.fIrma;
				case _: Assets.fBasic;
			}).render(s.bd, dialogue_txt, dialogue_x, dialogue_y, 100);
			dialogue_show = false;
		}
		
		// GUI
		s.bd.fillRect(0, 190, 320, 50, 0xFF998675);
		if (player_mode != PM_TALKING){
			for (i in 0...5){
				s.bd.fillRect(2 + 35 * i, 203, 34, 34, 0xFF736357);
				var mo:Bool = s.ms.getX() >= 3 + 35 * i && s.ms.getX() < 3 + 35 * i + 32
					&& s.ms.getY() >= 204 && s.ms.getY() < 236;
				if (!mo)
					s.bd.fillRect(3 + 35 * i, 204, 32, 32, 0xFF998675);
				if (i < inventory.length){
					items[inventory[i]].render(this, s.bd, 3 + 35 * i, 204);
					if (mo) player_moi = items[inventory[i]];
				}
			}
		}	
		switch (player_mode){
			case PM_FREE:
			if (player_moi == null){
				if (player_walkable)
					Assets.fBasic.render(s.bd, "Walk", 160 - FM.floor(4 * 2.5), 190, 100);
				return;
			}
			Assets.fBasic.render(s.bd, player_moi.name, 160 - FM.floor(player_moi.name.length * 2.5), 190, 100);
			
			case PM_TALKING:
			if (action_queue.length > 0) return;
			if (player_diacur == -1){
				player_mode = PM_FREE;
				return;
			}
			var ypos:Int = 190;
			for (inter in player_diatree[player_diacur]){
				if (s.ms.getX() >= 0 && s.ms.getX() < 320 && s.ms.getY() >= ypos && s.ms.getY() < ypos + 12){
					player_diasel = inter;
					s.bd.fillRect(0, ypos, 320, 13, 0xFF736357);
				} else {
					s.bd.fillRect(0, ypos + 12, 320, 1, 0xFF736357);
				}
				Assets.fBasic.render(s.bd, inter.name, 3, ypos, 100);
				ypos += 13;
			}
			
			case PM_SELECTING_INTERACTION:
			s.bd.fillRect(player_selx + 1, player_sely, player_selw - 2, player_selh, 0xFF998675);
			s.bd.fillRect(player_selx, player_sely, player_selw, 2, 0xFFC7B299);
			s.bd.fillRect(player_selx, player_sely + 2, player_selw, 1, 0xFF736357);
			s.bd.fillRect(player_selx, player_sely + player_selh - 3, player_selw, 1, 0xFF736357);
			s.bd.fillRect(player_selx + 1, player_sely, 1, player_selh, 0xFFC7B299);
			s.bd.fillRect(player_selx + 2, player_sely, 1, player_selh, 0xFF736357);
			s.bd.fillRect(player_selx + player_selw - 3, player_sely, 1, player_selh, 0xFF736357);
			s.bd.copyPixels(Assets.aGUI, player_selx, player_sely, 0, 16, 8, 8);
			s.bd.copyPixels(Assets.aGUI, player_selx + player_selw - 8, player_sely, 16, 16, 8, 8);
			s.bd.copyPixels(Assets.aGUI, player_selx, player_sely + player_selh - 8, 0, 32, 8, 8);
			s.bd.copyPixels(Assets.aGUI, player_selx + player_selw - 8, player_sely + player_selh - 8, 16, 32, 8, 8);
			var yp:Int = 0;
			for (ps in player_selection){
				if (s.ms.getX() >= player_selx + 4 && s.ms.getX() < player_selx + player_selw - 5
					&& s.ms.getY() >= player_sely + 4 + yp && s.ms.getY() < player_sely + 14 + yp){
					player_selmoi = ps;
					s.bd.fillRect(player_selx + 5, player_sely + 4 + yp, player_selw - 10, 1, 0xFF736357);
					s.bd.fillRect(player_selx + 5, player_sely + 14 + yp, player_selw - 10, 1, 0xFFC7B299);
					s.bd.fillRect(player_selx + 4, player_sely + 5 + yp, 1, 9, 0xFF736357);
					s.bd.fillRect(player_selx + player_selw - 5, player_sely + 5 + yp, 1, 9, 0xFFC7B299);
				} else {
					s.bd.fillRect(player_selx + 5, player_sely + 4 + yp, player_selw - 10, 1, 0xFFC7B299);
					s.bd.fillRect(player_selx + 5, player_sely + 14 + yp, player_selw - 10, 1, 0xFF736357);
					s.bd.fillRect(player_selx + 4, player_sely + 5 + yp, 1, 9, 0xFFC7B299);
					s.bd.fillRect(player_selx + player_selw - 5, player_sely + 5 + yp, 1, 9, 0xFF736357);
				}
				Assets.fSelection.render(s.bd, ps.name, player_selx + 6, player_sely + 5 + yp, 100);
				yp += 13;
			}
		}
	}
	
	override public function fClick(s:Stater, x:Int, y:Int):Void {
		switch (player_mode){
			case PM_FREE:
			player_pfatari = null;
			if (player_moi == null){
				if (player_walkable){
					var np = player_pfa.find(player_x + player_pfapath[0], player_y + player_pfapath[1],
						player_walkx, player_walky);
					if (player_pfapath.length > 0){
						player_pfapath.splice(2, player_pfapath.length - 2);
						player_pfapath = player_pfapath.concat(np);
					} else {
						player_pfapath = np;
					}
				}
				return;
			}
			player_selection = player_moi.getInteractions(this);
			player_mode = PM_SELECTING_INTERACTION;
			var w:Int = 0;
			var h:Int = 0;
			for (ps in player_selection){
				if (ps.name.length > w) w = ps.name.length;
				h++;
			}
			w *= 5;
			w += 12;
			h *= 13;
			h += 6;
			player_selx = x - (w >> 1);
			player_sely = y;
			if (player_selx < 0) player_selx = 0;
			if (player_selx + w >= 320) player_selx = 320 - w;
			if (player_sely < 0) player_sely = 0;
			if (player_sely + h >= 190) player_sely = 190 - h;
			player_selw = w;
			player_selh = h;
			
			case PM_TALKING:
			if (action_queue.length > 0 && action_queue[0].skippable){
				action_queue.shift();
				return;
			}
			if (player_diasel == null) return;
			player_diasel.exec(this);
			
			case PM_SELECTING_INTERACTION:
			if (player_selmoi == null){
				player_mode = PM_FREE;
				player_selection = null;
				return;
			}
			player_mode = PM_FREE;
			player_pfatari = null;
			if (player_selmoi.prox){
				if (player_x >= player_selmoi.proxx && player_x < player_selmoi.proxx + player_selmoi.proxw
					&& player_y >= player_selmoi.proxy && player_y < player_selmoi.proxy + player_selmoi.proxh){
					player_selmoi.exec(this);
				} else if (player_pfapath.length > 0
					&& player_x + player_pfapath[0] >= player_selmoi.proxx
					&& player_x + player_pfapath[0] < player_selmoi.proxx + player_selmoi.proxw
					&& player_y + player_pfapath[1] >= player_selmoi.proxy
					&& player_y + player_pfapath[1] < player_selmoi.proxy + player_selmoi.proxh){
					player_pfatari = player_selmoi;
				} else {
					var np = player_pfa.find(player_x + player_pfapath[0], player_y + player_pfapath[1],
						player_walkx, player_walky);
					if (player_pfapath.length > 0){
						player_pfapath.splice(2, player_pfapath.length - 2);
						player_pfapath = player_pfapath.concat(np);
					} else {
						player_pfapath = np;
					}
					
					var np = if (player_selmoi.proxw == 1 && player_selmoi.proxh == 1){
						player_pfa.find(player_x + player_pfapath[0], player_y + player_pfapath[1],
							player_selmoi.proxx, player_selmoi.proxy);
					} else {
						var tgt:Array<Array<Int>> = [];
						for (y in 0...player_selmoi.proxh) for (x in 0...player_selmoi.proxw){
							tgt.push([player_selmoi.proxx + x, player_selmoi.proxy + y]);
						}
						player_pfa.findAny(player_x + player_pfapath[0], player_y + player_pfapath[1], tgt);
					};
					
					if (player_pfapath.length > 0){
						player_pfapath.splice(2, player_pfapath.length - 2);
						player_pfapath = player_pfapath.concat(np);
						if (np.length > 0) player_pfatari = player_selmoi;
					} else {
						player_pfapath = np;
					}
				}
			} else {
				player_selmoi.exec(this);
			}
			
			case PM_ACTION:
			if (action_queue.length <= 0) return;
			if (action_queue[0].skippable){
				action_queue.shift();
				if (action_queue.length == 0) player_mode = PM_FREE;
			}
		}
	}
}