package game;

import sk.thenet.plustd.FM;
import sk.thenet.gamestd.bmp.BitmapData;

class SFacility extends Screen {
	public var pos_x:Float = 0;
	public var pos_y:Float = 380;
	public var mach:InMachine;
	public var visited:Bool = false;
	
	public function new(){
		super("Facility");
		playerSmall = true;
		bg = Assets.aSFacility;
		fg = Assets.aSFacility;
		fg_x = 640;
		hasfg = true;
		
		initWalkmap(340, 165, 25, 34,
			10, -10,
			5, 5,
		0, 6);
		walkmap_speedmul = 2;
		walkmapScreenBlock(6, 6, 12, 12, true);
		walkmapScreenBlock(0, 25, 10, 9, true);
		walkmapScreenBlock(15, 25, 10, 9, true);
		walkmapScreenBlock(16, 18, 2, 2, true);
		
		mach = new InMachine(this);
		interactives = [
			mach,
			new InConsole(this),
			new InExit(this)
		];
	}
	
	override public function tick(gs:GSGame):Void {
		for (inter in interactives){
			if (!inter.isActor) continue;
			var act = cast(inter, Actor);
			act.scrz = FM.floor(act.x + act.y);
		}
		if (!visited){
			gs.action_queue.push(new Action.Wait(60));
			gs.action_queue.push(Action.Dialogue.playerSay(
				"What is this place?"
			));
			visited = true;
		}
		var ppos = gs.player.getScreenPos();
		pos_x = (pos_x * 14 + ppos.x) / 15;
		pos_y = (pos_y * 14 + ppos.y) / 15;
		bg_x = FM.floor(pos_x) - 160;
		bg_x = (bg_x < 0 ? 0 : (bg_x > 320 ? 320 : bg_x));
		bg_y = FM.floor(pos_y) - 140;
		bg_y = (bg_y < 0 ? 0 : (bg_y > 240 ? 240 : bg_y));
		fg_x = bg_x + 640;
		fg_y = bg_y;
	}
}

class InExit extends Interactive {
	public function new(nscreen:Screen){
		super("Exit", nscreen);
		hasint = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		var jar:Bool = false;
		for (i in 0...gs.screens[2].interactives.length){
			if (gs.screens[2].interactives[i].isJar){
				jar = true;
				break;
			}
		}
		
		return jar ? [
			new Interaction("Leave", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.fadecol = 0;
					gs.fadetar = 100;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(50));
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.switchScreen(0, 17, 0);
					gs.fadetar = 0;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(40));
			}, true, 13, 33, 1, 1)
		] : [
			new Interaction("Leave", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"I can't leave now."
				));
			}, false)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (80 - sx) && mx <= (150 - sx) && my > (380 - sy) && my <= (420 - sy);
	}
}

class InMachine extends Interactive {
	public var jar0:Bool = false;
	public var jar1:Bool = false;
	
	public function new(nscreen:Screen){
		super("Machine?", nscreen);
		hasint = true;
		//nscreen.walkmapBlock(15, 0, 5, 1, true);
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		var act = [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"Looks creepy...\nI can't tell what it does."
				));
				if (jar0 || jar1) return;
				gs.action_queue.push(Action.Dialogue.playerSay(
					"There are some slots in the front,\nmaybe something should go in?"
				));
			}, false)
		];
		if (jar0){
			act.push(
				new Interaction("Unplug red jar", 0, function(gs:GSGame){
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(new Action.Wait(20));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"I can't take it out, it's held securely."
					));
				}, true, 5, 14, 1, 1)
			);
		}
		if (jar1){
			act.push(
				new Interaction("Unplug green jar", 0, function(gs:GSGame){
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(new Action.Wait(20));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"I can't take it out, it's held securely."
					));
				}, true, 5, 14, 1, 1)
			);
		}
		if (jar0 != jar1){
			act.push(
				new Interaction("Examine " + (jar0 ? "red" : "green") + " jar", 0, function(gs:GSGame){
					gs.action_queue.push(Action.Dialogue.playerSay(
						"It is glowing and bubbling a bit inside."
					));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"I think it's the brain of the machine."
					));
				}, false)
			);
		} else if (jar0 && jar1){
			act.push(
				new Interaction("Examine jars", 0, function(gs:GSGame){
					gs.action_queue.push(Action.Dialogue.playerSay(
						"They are glowing and bubbling a bit inside."
					));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"I think they're the brain of the machine."
					));
				}, false)
			);
		}
		if (gs.inventory.indexOf(0) >= 0){
			act.push(
				new Interaction("Plug in red jar", 0, function(gs:GSGame){
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(Action.Dialogue.playerSay(
						"Let's see if this fits in..."
					));
					gs.action_queue.push(new Action.Wait(20));
					gs.action_queue.push(new Action.LoseItem(0));
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Execute(function(gs:GSGame){
						jar0 = true;
						Noises.shortClick();
						return true;
					}));
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"Perfect fit."
					));
				}, true, 5, 14, 1, 1)
			);
		}
		if (gs.inventory.indexOf(1) >= 0){
			act.push(
				new Interaction("Plug in green jar", 0, function(gs:GSGame){
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(Action.Dialogue.playerSay(
						"Let's see if this fits in..."
					));
					gs.action_queue.push(new Action.Wait(20));
					gs.action_queue.push(new Action.LoseItem(1));
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Execute(function(gs:GSGame){
						jar1 = true;
						Noises.shortClick();
						return true;
					}));
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"Perfect fit."
					));
				}, true, 5, 14, 1, 1)
			);
		}
		return act;
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (264 - sx) && mx <= (408 - sx) && my > (224 - sy) && my <= (301 - sy);
	}
	
	override public function render(gs:GSGame, bd:BitmapData, sx:Int, sy:Int):Void {
		bd.copyPixels(Assets.aSFacility, 252 - sx, 240 - sy - 160, 0, 480, 170, 257);
		if (jar1) bd.copyPixels(Assets.aSFacility, 252 - sx + 16, 240 - sy - 15, 184 + 36, 520, 36, 48);
		if (jar0) bd.copyPixels(Assets.aSFacility, 252 - sx + 16, 240 - sy - 15, 184, 520, 36, 48);
	}
	
	override public inline function getZ():Int {
		return 24;
	}
}

class InConsole extends Interactive {
	public function new(nscreen:Screen){
		super("Console", nscreen);
		hasint = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		var jar0 = cast(screen, SFacility).mach.jar0;
		var jar1 = cast(screen, SFacility).mach.jar1;
		return [
			new Interaction("Activate console", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.player.runaway = 1;
					Noises.musicBootup(jar0 && jar1);
					return true;
				}));
				gs.action_queue.push(new Action.Dialogue(
					264 - screen.bg_x, 224 - screen.bg_y, 1,
					"BOOT SEQUENCE",
					0, true
				));
				gs.action_queue.push(new Action.Wait(20));
				
				if (!jar0 && !jar1){
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"< DATA MISSING! >",
						0, true
					));
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"jar0 status: unk\njar1 status: unk\nABORT",
						0, true
					));
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Execute(function(gs:GSGame){
						gs.player.runaway = 0;
						return true;
					}));
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"I think it's missing something."
					));
				} else if (jar0 != jar1){
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"jar0 status: " + (jar0 ? "OK" : "unk") + "\njar1 status: " + (jar1 ? "OK" : "unk") + "\nCOREDUMP",
						0, true
					));
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"Primary objectives:\n 1.  Collect data\n 2.  Archive data\n 3. ${{ }|! CORRUPTED",
						0, true
					));
					if (gs.player.vr){
						gs.action_queue.push(new Action.Dialogue(
							264 - screen.bg_x, 224 - screen.bg_y, 1,
							"Secondary agent deployed.",
							0, true
						));
						gs.action_queue.push(new Action.Wait(10));
						gs.action_queue.push(new Action.Execute(function(gs:GSGame){
							gs.player.runaway = 0;
							return true;
						}));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"I guess I am its 'secondary agent'..."
						));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"Whatever that means."
						));
					} else {
						gs.action_queue.push(new Action.Dialogue(
							264 - screen.bg_x, 224 - screen.bg_y, 1,
							"Evaluating best course of action . . .",
							0, true
						));
						gs.action_queue.push(new Action.Wait(20));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"Collect data? Archive?\nWhat is this machine for?"
						));
						gs.action_queue.push(new Action.Wait(10));
						gs.action_queue.push(new Action.Dialogue(
							264 - screen.bg_x, 224 - screen.bg_y, 1,
							"Secondary agent required.\nModifying user status . . .",
							0, true
						));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"Wait ... no...!"
						));
						for (i in 2...10){
							gs.action_queue.push(new Action.Execute(function(gs:GSGame){
								gs.player.runaway = i;
								return true;
							}));
							if (i == 5){
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.fadecol = 0xFF11DD;
									gs.fade = 100;
									gs.fadetar = 100;
									Noises.shootDown();
									return true;
								}));
								gs.action_queue.push(new Action.Wait(3));
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.fade = 0;
									gs.fadetar = 0;
									return true;
								}));
							}
							if (i == 8){
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.fadecol = 0x37444F;
									gs.fadetar = 100;
									return true;
								}));
							}
							gs.action_queue.push(new Action.Wait(10));
						}
						gs.action_queue.push(new Action.Wait(100));
						gs.action_queue.push(new Action.Execute(function(gs:GSGame){
							gs.player.runaway = 0;
							gs.player.vr = true;
							gs.inventory.unshift(2);
							gs.fadetar = 0;
							return true;
						}));
						gs.action_queue.push(new Action.Wait(50));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"Ugh ... my head."
						));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"It's attached a VR headset to my head!"
						));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"What the hell is going on?"
						));
						gs.action_queue.push(new Action.Wait(20));
						gs.action_queue.push(new Action.Execute(function(gs:GSGame){
							gs.vr_disptar = 100;
							gs.vr_orig = VR.renderObjective("Explore :TOWN >");
							return true;
						}));
						gs.action_queue.push(new Action.Wait(170));
						gs.action_queue.push(new Action.Execute(function(gs:GSGame){
							gs.vr_disptar = 0;
							return true;
						}));
						gs.action_queue.push(new Action.Wait(40));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"Seems very damaged."
						));
						gs.action_queue.push(Action.Dialogue.playerSay(
							"Does it need something from the town?"
						));
					}
				} else if (jar0 && jar1){
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"jar0 status: OK\njar1 status: OK\nCOREDUMP",
						0, false
					));
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"Primary objectives:\n 1.  Collect data\n 2.  Archive data\n 3.  Exterminate",
						0, false
					));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"What?!"
					));
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"Secondary agent completed primary objectives.\nData point: <Irma ERWITT>",
						0, false
					));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"Exterminated?"
					));
					gs.action_queue.push(new Action.Dialogue(
						264 - screen.bg_x, 224 - screen.bg_y, 1,
						"Shutting off secondary agent",
						0, false
					));
					gs.action_queue.push(Action.Dialogue.playerSay(
						"But ..."
					));
					gs.action_queue.push(new Action.Execute(function(gs:GSGame){
						gs.fadecol = 0xFF11DD;
						gs.fade = 100;
						gs.fadetar = 100;
						Noises.shootDown();
						return true;
					}));
					gs.action_queue.push(new Action.Wait(3));
					gs.action_queue.push(new Action.Execute(function(gs:GSGame){
						Main.st.switchTo("ending2");
						return true;
					}));
				}
			}, true, 17, 20, 1, 1)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (311 - sx) && mx <= (310 + 28 - sx) && my > (311 - sy) && my <= (311 + 38 - sy);
	}
	
	override public function render(gs:GSGame, bd:BitmapData, sx:Int, sy:Int):Void {
		bd.copyPixels(Assets.aSFacility, 310 - sx, 311 - sy, 184, 480, 28, 38);
	}
	
	override public inline function getZ():Int {
		return 34;
	}
}