package game;

import sk.thenet.plustd.FM;
import sk.thenet.gamestd.bmp.BitmapData;

class SGarage extends Screen {
	public var pos:Float = 0;
	public var para_bg:InGarageBG;
	public var jar:Bool = true;
	
	public function new(){
		super("Garage");
		bg = Assets.aSGarage;
		
		initWalkmap(80, 145, 28, 3,
			20, 0,
			0, 16,
		100, 11);
		walkmapScreenBlock(13, 0, 2, 1, true);
		
		para_bg = new InGarageBG(this);
		interactives = [
			para_bg,
			new InHoverbike(this),
			new InIrma(this),
			new InRadio(this),
			new InGaragePath(this),
			new InJar(this)
		];
		
		playerMedium = true;
	}
	
	override public function init():Void {
		music = Noises.loopGarage();
		for (inter in interactives){
			if (inter.isRadio){
				cast(inter, InRadio).isOn = true;
				break;
			}
		}
	}
	
	override public function tick(gs:GSGame):Void {
		var ppos = gs.player.getScreenPos();
		pos = (pos * 14 + ppos.x) / 15;
		bg_x = FM.floor(pos) - 160;
		bg_x = (bg_x < 0 ? 0 : (bg_x > 320 ? 320 : bg_x));
		para_bg.pos = -(bg_x >> 1);
	}
}

class InJar extends Interactive {
	public function new(nscreen:Screen){
		super("Red datajar", nscreen);
		hasint = true;
		isJar = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		return [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"It's another datajar."
				));
			}, false),
			new Interaction("Take", 0, function(gs:GSGame){
				gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2, 
					"If you can fix the starter,\nI'll let you take it."
				));
			}, false)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (451 - sx) && mx <= (475 - sx) && my > 112 && my <= 144;
	}
	
	override public function render(gs:GSGame, bd:BitmapData, sx:Int, sy:Int):Void {
		bd.copyPixels(Assets.aSGarage, 451 - sx, 112 - sy - 160, 568, 216, 24, 32);
	}
	
	override public function getZ():Int { return 0; }
}

class InGaragePath extends Interactive {
	public function new(nscreen:Screen){
		super("Path", nscreen);
		hasint = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		return [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"The path leads back to the facility."
				));
			}, false),
			new Interaction("Follow path", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.fadecol = 0;
					gs.fadetar = 100;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(50));
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.switchScreen(0, 25, 1);
					gs.fadetar = 0;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(40));
			}, true, 0, 1, 1, 1)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (0 - sx) && mx <= (60 - sx) && my > 20 && my <= 130;
	}
}

class InGarageBG extends Interactive {
	public var pos:Int = 0;
	
	public function new(nscreen:Screen){
		super("BG", nscreen);
	}
	
	override public function render(gs:GSGame, bd:BitmapData, sx:Int, sy:Int):Void {
		bd.copyPixels(Assets.aSGarage, pos, 0, 0, 240, 640, 240);
	}
	
	override public inline function getZ():Int {
		return -1;
	}
}

class InRadio extends Interactive {
	public var isOn:Bool = true;
	public var newsTrack:Int = 0;
	public var newsBike:Bool = false;
	
	public function new(nscreen:Screen){
		super("Radio", nscreen);
		hasint = true;
		isRadio = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		return [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"An old radio, blasting old music."
				));
			}, false)
		].concat(isOn ? [
			new Interaction("Listen", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				switch (newsTrack){
					case 0:
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Archaeology today!"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Scientist have uncovered an artifact...\n alegedly owned by Genghis Khan ...\n"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"... it is said to bring misfortune\n... anybody who mentions its name: 'Dothranas'"
					));
					case 1:
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"And for weather today ..."
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"We are experiencing severe\nsharknados all over Florida."
					));
					case 2:
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Mike, what's new in sports?"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Well, Chuck, I ask you the question..."
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"What is sports, exactly?"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Is it an urge to be the best?\nDo we want fame and fortune?"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"I don't know, but it seems\nthat people don't care anyway."
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Thank you, Mike."
					));
					case 3:
					gs.player_mode = GSGame.PM_ACTION;
					gs.action_queue.push(new Action.Wait(10));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"DIY hoverbike news"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"Have you looked at your\nbike's bikeburator recently?"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"No? What are you waiting for?!"
					));
					gs.action_queue.push(new Action.Dialogue(260 - screen.bg_x, 70, 0, 
						"As any real DIY hoverbiker\nknows, it's always the trillian circuit."
					));
					newsBike = true;
				}
				newsTrack++;
				newsTrack %= 4;
			}, false),
			new Interaction("Turn off", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Wait(10));
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					isOn = false;
					screen.music.stop();
					return true;
				}));
				gs.action_queue.push(Action.Dialogue.playerSay(
					"That's better."
				));
				gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2, 
					"Hey, I was listening to that!"
				));
			}, true, 13, 1, 1, 1),
		] : [
			new Interaction("Turn on", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Wait(10));
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					isOn = true;
					screen.music = Noises.loopGarage();
					return true;
				}));
				gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2, 
					"Cheers."
				));
			}, true, 13, 1, 1, 1)
		]);
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (334 - sx) && mx <= (363 - sx) && my > 132 && my <= 151;
	}
}

class InHoverbike extends Interactive {
	public function new(nscreen:Screen){
		super("Hoverbike", nscreen);
		hasint = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		return [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"A shiny hoverbike."
				));
				gs.action_queue.push(Action.Dialogue.playerSay(
					"It's been a long time since\nI rode one of these."
				));
				gs.action_queue.push(new Action.Wait(10));
				gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2, 
					"If you can fix the starter,\nI'll let you ride it."
				));
			}, false)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (242 - sx) && mx <= (294 - sx) && my > 77 && my <= 136;
	}
}

class InIrma extends Interactive {
	public function new(nscreen:Screen){
		super("Irma", nscreen);
		hasint = true;
	}
	
	public var talkIrma:Bool = false;
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		var newsBike:Bool = false;
		for (inter in screen.interactives){
			if (inter.isRadio){
				if (cast(inter, InRadio).newsBike) newsBike = true;
				break;
			}
		}
		return [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"A woman, admiring her hoverbike, I guess?"
				));
				gs.action_queue.push(Action.Dialogue.playerSay(
					"Doesn't look too friendly."
				));
				gs.action_queue.push(new Action.Wait(20));
				gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
					"I can hear you, you know..."
				));
			}, false),
			new Interaction("Talk", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_TALKING;
				gs.player_diacur = 0;
				gs.player_diatree = [
					[
						new Interaction("Where am I?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Where am I?"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"This used to be a town\ncalled Peasant's Grotto."
							));
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Used to?"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"As you can see, it's deserted.\nUsed to be a hunting town..."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"But then there was nothing\nto hunt anymore."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"I stayed here cause I like\nthe fresh air and solitude."
							));
							gs.player_diacur = -1;
						})
					].concat(talkIrma ? [
						new Interaction("So what do you do all day?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"So what do you do all day?"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"Build things, fix things,\nride my hoverbike."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"See this beauty? I built it myself."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"Except now I can't figure out\nwhat's wrong with the starter."
							));
							gs.player_diacur = 2;
						})
					] : [
						new Interaction("Who are you?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Who are you?"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"I'm Irma, the local mechanic. Who are you?"
							));
							gs.player_diacur = 1;
						})
					]).concat(newsBike ? [
						new Interaction("About the bike starter...", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"About the bike starter..."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"Yes?"
							));
							gs.player_diacur = 2;
						})
					] : []),
					[ // who am I?
						new Interaction("I don't know, I lost my memory.", 0, function(gs:GSGame){
							talkIrma = true;
							gs.player_diatree[0][1] = new Interaction("So what do you do all day?", 0, function(gs:GSGame){
								gs.action_queue.push(Action.Dialogue.playerSay(
									"So what do you do all day?"
								));
								gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
									"Build things, fix things,\nride my hoverbike."
								));
								gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
									"See this beauty? I built it myself."
								));
								gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
									"Except now I can't figure out\nwhat's wrong with the starter."
								));
								gs.player_diacur = 2;
							});
							gs.action_queue.push(Action.Dialogue.playerSay(
								"I don't know, I lost my memory."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"I see. Have a look around..."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"I'm sure the memories will come flooding in."
							));
							gs.player_diacur = 0;
						}),
						new Interaction("None of your business.", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"None of your business."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"Eh."
							));
							gs.player_diacur = -1;
						})
					],
					[ // starter
						new Interaction("Have you tried rerouting the navicomp power?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Have you tried rerouting the power from ..."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"Of course I have."
							));
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Oh."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"The degausscene circuit seems okay."
							));
							gs.action_queue.push(Action.Dialogue.playerSay(
								"That's good."
							));
						}),
						new Interaction("Have you tried turning it on and off again?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Have you tried turning it on and off again?"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"It's the *starter*, it's what\nturns the bike on and off."
							));
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Of course ..."
							));
						}),
						new Interaction("Have you cleaned the yamato core recently?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Have you cleaned the yamato ..."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"Duh, it's the first thing I tried."
							));
						}),
						(newsBike ? new Interaction("Have you checked the trillian circuit?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Have you checked the trillian circuit?"
							));
							gs.action_queue.push(new Action.Wait(50));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"By god, you're right!\nThat must be it!"
							));
							gs.action_queue.push(new Action.Execute(function(gs:GSGame){
								screen.music.stop();
								gs.fadecol = 0;
								gs.fadetar = 100;
								return true;
							}));
							gs.action_queue.push(new Action.Wait(50));
							gs.action_queue.push(new Action.Execute(function(gs:GSGame){
								gs.fadetar = 0;
								return true;
							}));
							gs.action_queue.push(new Action.Wait(40));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"It worked!"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"So what's it gonna be?\nA ride, or something else of mine?"
							));
							gs.player_diacur = 3;
						}) : new Interaction("Have you considered buying a new bike?", 0, function(gs:GSGame){
							gs.action_queue.push(Action.Dialogue.playerSay(
								"Have you considered buying a new bike?"
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"No. You don't know much\nabout bikes, do you?"
							));
							gs.action_queue.push(Action.Dialogue.playerSay(
								"I guess not."
							));
							gs.player_diacur = -1;
						}))
					],
					[ // final
						new Interaction("I want to take a ride.", 0, function(gs:GSGame){
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"It's all yours."
							));
							if (gs.player.vr){
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.vr_orig = VR.renderEnding("FAILURE\nSecondary\n  agent\n  faulty\n\nExecute-");
									gs.vr_disptar = 100;
									return true;
								}));
								gs.action_queue.push(new Action.Wait(400));
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.fadecol = 0;
									gs.fadetar = 100;
									gs.vr_disptar = 0;
									return true;
								}));
								gs.action_queue.push(new Action.Wait(200));
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.fadecol = 0xFF11DD;
									gs.fade = 100;
									gs.fadetar = 100;
									Noises.shootDown();
									return true;
								}));
								gs.action_queue.push(new Action.Wait(3));
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									Main.st.switchTo("ending1");
									return true;
								}));
							} else {
								gs.action_queue.push(new Action.Wait(50));
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									gs.fadecol = 0;
									gs.fadetar = 100;
									return true;
								}));
								gs.action_queue.push(new Action.Wait(400));
								gs.action_queue.push(new Action.Execute(function(gs:GSGame){
									Main.st.switchTo("ending3");
									return true;
								}));
							}
						}),
						new Interaction("I would like that datajar.", 0, function(gs:GSGame){
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"It's yours."
							));
							gs.action_queue.push(new Action.Dialogue(280 - screen.bg_x, 50, 2,
								"I will taKe a ridE myself.",
							0, true));
							gs.action_queue.push(new Action.Execute(function(gs:GSGame){
								gs.inventory.push(0);
								for (i in 0...screen.interactives.length){
									if (screen.interactives[i].isJar){
										screen.interactives.splice(i, 1);
										break;
									}
								}
								return true;
							}));
							gs.action_queue.push(new Action.Execute(function(gs:GSGame){
								gs.fadecol = 0;
								gs.fadetar = 100;
								return true;
							}));
							gs.action_queue.push(new Action.Wait(100));
							gs.action_queue.push(new Action.Execute(function(gs:GSGame){
								gs.fadetar = 0;
								gs.switchScreen(1, 17, 20);
								return true;
							}));
							gs.action_queue.push(new Action.Wait(40));
							gs.player_diacur = -1;
						}),
					]
				];
			}, false)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (304 - sx) && mx <= (324 - sx) && my > 74 && my <= 132;
	}
}