package game;

import sk.thenet.plustd.FM;
import sk.thenet.gamestd.bmp.BitmapData;

class SJungle extends Screen {
	public var pos:Float = 0;
	
	public function new(){
		super("Jungle");
		bg = Assets.aSJungle;
		fg = Assets.aSJungle;
		fg_y = 190;
		hasfg = true;
		
		initWalkmap(80, 122, 28, 4,
			20, 0,
			0, 18,
		100, 11);
		walkmapScreenBlock(0, 0, 15, 1, true);
		walkmapScreenBlock(20, 0, 100, 1, true);
		
		interactives = [
			new InJungleDoor(this),
			new Interactive("fg", this),
			new InJunglePath(this)
		];
	}
	
	override public function init():Void {
		music = Noises.loopJungle();
	}
	
	override public function tick(gs:GSGame):Void {
		var ppos = gs.player.getScreenPos();
		pos = (pos * 14 + ppos.x) / 15;
		bg_x = FM.floor(pos) - 160;
		bg_x = (bg_x < 0 ? 0 : (bg_x > 320 ? 320 : bg_x));
		fg_x = FM.floor(bg_x * 1.5);
	}
}

class InJunglePath extends Interactive {
	public function new(nscreen:Screen){
		super("Path", nscreen);
		hasint = true;
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		return [
			new Interaction("Examine", 0, function(gs:GSGame){
				gs.action_queue.push(Action.Dialogue.playerSay(
					"A path in the jungle, it's barely visible."
				));
			}, false),
			new Interaction("Follow path", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.fadecol = 0;
					gs.fadetar = 100;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(50));
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.switchScreen(2, 1, 2);
					gs.fadetar = 0;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(40));
			}, true, 27, 1, 1, 1)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (610 - sx) && mx <= (640 - sx) && my > 20 && my <= 190;
	}
}

class InJungleDoor extends Interactive {
	var pos:Int = 0;
	var opening:Bool = false;
	
	public function new(nscreen:Screen){
		super("Door", nscreen);
		hasint = true;
		nscreen.walkmapBlock(15, 0, 5, 1, true);
	}
	
	override public function getInteractions(gs:GSGame):Array<Interaction> {
		if (opening && pos < 60) return [];
		if (opening) return [
			new Interaction("Enter", 0, function(gs:GSGame){
				gs.player_mode = GSGame.PM_ACTION;
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.fadecol = 0;
					gs.fadetar = 100;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(50));
				gs.action_queue.push(new Action.Execute(function(gs:GSGame){
					gs.switchScreen(1, 13, 32);
					gs.fadetar = 0;
					return true;
				}));
				gs.action_queue.push(new Action.Wait(40));
			}, true, 17, 0, 1, 1)
		];
		return [
			new Interaction("Open door", 0, function(gs:GSGame){
				opening = true;
				Noises.jungledoorOpen();
			}, true, 15, 1, 5, 1)
		];
	}
	
	override public function isMouseover(mx:Int, my:Int, sx:Int, sy:Int):Bool {
		return mx > (370 - sx) && mx <= (473 - sx) && my > 21 && my <= 129;
	}
	
	override public function render(gs:GSGame, bd:BitmapData, sx:Int, sy:Int):Void {
		if (opening){
			if (pos < 60){
				hasint = false;
				pos++;
			} else if (pos == 60){
				hasint = true;
				screen.walkmapBlock(15, 0, 5, 1, false);
				pos++;
			}
		}
		bd.copyPixels(Assets.aSJungle, 370 - sx, 21, 654, 0, 103, 108);
		bd.copyPixels(Assets.aSJungle, 370 - sx - FM.floor(pos >> 2), 21 + (pos << 1), 654 + 103, 0, 111, 108);
	}
	
	override public inline function getZ():Int {
		return -1;
	}
}