package game;

import sk.thenet.plustd.audio.music.MusicManager;
import sk.thenet.gamestd.bmp.BitmapData;
import sk.thenet.gamestd.pathfind.PFAstar;

class Screen {
	public var name:String;
	public var playerSmall:Bool = false;
	public var playerMedium:Bool = false;
	public var bg:BitmapData;
	public var fg:BitmapData;
	public var bg_x:Int;
	public var bg_y:Int;
	public var fg_x:Int;
	public var fg_y:Int;
	public var hasfg:Bool;
	public var interactives:Array<Interactive>;
	public var music:MusicManager;
	
	public var walkmap:Array<Walkpoint>;
	public var walkmap_w:Int;
	public var walkmap_h:Int;
	public var walkmap_x:Int;
	public var walkmap_y:Int;
	public var walkmap_xx:Float;
	public var walkmap_xy:Float;
	public var walkmap_yx:Float;
	public var walkmap_yy:Float;
	public var walkmap_backadd:Int;
	public var walkmap_dist:Int;
	public var walkmap_speedmul:Float = 1;
	
	public var pfa:PFAstar;
	
	public function new(nname:String){
		name = nname;
		interactives = [];
		bg_x = 0;
		bg_y = 0;
		fg_x = 0;
		fg_y = 0;
		hasfg = false;
	}
	
	public function initWalkmap(nx:Int, ny:Int, nw:Int, nh:Int,
		nxx:Float, nxy:Float, nyx:Float, nyy:Float, nba:Int, ndist:Int):Void {
		walkmap = [];
		walkmap_x = nx;
		walkmap_y = ny;
		walkmap_w = nw;
		walkmap_h = nh;
		walkmap_xx = nxx;
		walkmap_xy = nxy;
		walkmap_yx = nyx;
		walkmap_yy = nyy;
		walkmap_backadd = nba;
		walkmap_dist = ndist;
		for (i in 0...(nw * nh)){
			walkmap[i] = new Walkpoint();
		}
		updateBacks();
	}
	
	public function walkmapBlock(nx:Int, ny:Int, nw:Int, nh:Int, nbl:Bool):Void {
		//trace("a", nx, ny, nw, nh, nbl);
		var i:Int = 0;
		for (y in 0...walkmap_h) for (x in 0...walkmap_w){
			//trace("i");
			if (x >= nx && x < nx + nw && y >= ny && y < ny + nh){
				walkmap[i].blocked += (nbl ? 1 : -1);
				//trace("A");
			}
			i++;
		}
		updateBacks();
	}
	
	public function walkmapScreenBlock(nx:Int, ny:Int, nw:Int, nh:Int, nbl:Bool):Void {
		var i:Int = 0;
		for (y in 0...walkmap_h) for (x in 0...walkmap_w){
			if (x >= nx && x < nx + nw && y >= ny && y < ny + nh) walkmap[i].screen_block = nbl;
			i++;
		}
		updateBacks();
	}
	
	public function updatePFA(npfa:PFAstar):Void {
		pfa = npfa;
		updateBacks();
	}
	
	public function updateBacks():Void {
		var bm:Array<Int> = [];
		for (x in 0...walkmap_w){ bm[x] = walkmap_h - 1; }
		var y:Int = walkmap_h - 2;
		while (y >= 0){
			for (x in 0...walkmap_w){
				if (walkmap[y * walkmap_w + x].isWalkable()) bm[x] = y;
			}
			y--;
		}
		var i:Int = 0;
		var nm = new Array<Int>();
		for (y in 0...walkmap_h) for (x in 0...walkmap_w){
			walkmap[i].back = (bm[x] == y);
			nm[i] = (walkmap[i].isWalkable() ? 1 : 0);
			i++;
		}
		if (pfa != null){
			pfa.init(nm, walkmap_w, walkmap_h);
		}
	}
	
	public function tick(gs:GSGame):Void {}
	
	public function init():Void {}
	
	public function uninit():Void {
		if (music != null){
			music.stop();
			music = null;
		}
	}
}