package sk.thenet.gamestd.bmp;

import sk.thenet.plustd.FM;
import haxe.ds.Vector;

class BitmapFont {
	var bd:BitmapData;
	var bdw:Int; // chars per line of bd
	var bdo:Int; // bd charcode offset
	var chw:Int; // char width
	var chh:Int; // char height
	var cc:UInt; // colour
	
	public function new(nbd:BitmapData, nbdw:Int, nbdo:Int, nchw:Int, nchh:Int, nc:UInt = 0, ngc:UInt = 0){
		bd = nbd;
		bdw = nbdw;
		bdo = nbdo;
		chw = nchw;
		chh = nchh;
		if (ngc != 0){
			glow(nc, ngc);
		} else if (nc != 0){
			recolour(nc);
		}
	}
	
	public function recolour(c:UInt):Void {
		cc = c;
		var v = bd.getFullVector();
		for (i in 0...v.length){
			v[i] = (v[i] & 0xFF000000) | (c & 0xFFFFFF);
		}
		bd.setFullVector(v);
	}
	
	public function glow(c:UInt, gc:UInt):Void {
		var bdh:Int = FM.round(bd.height / chh);
		var nbd:BitmapData = new BitmapData(bdw * (chw + 2), bdh * (chh + 2), 0);
		for (y in 0...bdh){
			for (x in 0...bdw){
				for (sy in 0...chh){
					for (sx in 0...chw){
						if (bd.get(x * chw + sx, y * chh + sy) & 0xFF000000 != 0){
							nbd.set(x * (chw + 2) + sx, y * (chh + 2) + sy, gc);
							nbd.set(x * (chw + 2) + sx + 1, y * (chh + 2) + sy, gc);
							nbd.set(x * (chw + 2) + sx + 2, y * (chh + 2) + sy, gc);
							nbd.set(x * (chw + 2) + sx, y * (chh + 2) + sy + 1, gc);
							nbd.set(x * (chw + 2) + sx + 2, y * (chh + 2) + sy + 1, gc);
							nbd.set(x * (chw + 2) + sx, y * (chh + 2) + sy + 2, gc);
							nbd.set(x * (chw + 2) + sx + 1, y * (chh + 2) + sy + 2, gc);
							nbd.set(x * (chw + 2) + sx + 2, y * (chh + 2) + sy + 2, gc);
						}
					}
				}
				for (sy in 0...chh){
					for (sx in 0...chw){
						if (bd.get(x * chw + sx, y * chh + sy) & 0xFF000000 != 0){
							nbd.set(x * (chw + 2) + sx + 1, y * (chh + 2) + sy + 1, c);
						}
					}
				}
			}
		}
		chw += 2;
		chh += 2;
		bd = nbd;
	}
	
	public function render(tbd:BitmapData, txt:String, sx:Int, sy:Int, mw:Int):Void {
		var x:Int = sx;
		var xc:Int = 0;
		var y:Int = sy;
		for (ch in 0...txt.length){
			if (txt.charAt(ch) == "\n"){
				x = sx;
				xc = 0;
				y += chh;
			} else {
				if (txt.charAt(ch) != " " && xc < mw){
					if (txt.charCodeAt(ch) >= bdo){
						tbd.copyPixels(bd, x, y, ((txt.charCodeAt(ch) - bdo) % bdw) * chw, FM.floor((txt.charCodeAt(ch) - bdo) / bdw) * chh, chw, chh);
					}
				}
				x += chw;
				xc++;
			}
		}
	}
	
	public function renderColour(tbd:BitmapData, txt:String, sx:Int, sy:Int, mw:Int, rc:UInt):Void {
		var oc:UInt = cc;
		recolour(rc);
		var x:Int = sx;
		var xc:Int = 0;
		var y:Int = sy;
		for (ch in 0...txt.length){
			if (txt.charAt(ch) == "\n"){
				x = sx;
				xc = 0;
				y += chh;
			} else {
				if (txt.charAt(ch) != " " && xc < mw){
					if (txt.charCodeAt(ch) >= bdo){
						tbd.copyPixels(bd, x, y, ((txt.charCodeAt(ch) - bdo) % bdw) * chw, FM.floor((txt.charCodeAt(ch) - bdo) / bdw) * chh, chw, chh);
					}
				}
				x += chw;
				xc++;
			}
		}
		recolour(oc);
	}
}