package sk.thenet.gamestd.pathfind;

import haxe.ds.Vector;
import sk.thenet.plustd.FM;

class PFAstar {
	public var m:Vector<PFAstarTile>;
	public var w:Int;
	public var h:Int;
	
	public function new(){}
	
	public function init(nm:Array<Int>, nw:Int, nh:Int):Void {
		w = nw;
		h = nh;
		m = new Vector<PFAstarTile>(w * h);
		var x:Int = 0;
		var y:Int = 0;
		var i:Int = 0;
		for (t in nm){
			m[i] = new PFAstarTile((t & 1) == 1, (t & 2) == 2, x, y);
			i++;
			x++;
			if (x >= w){
				x = 0;
				y++;
			}
		}
	}
	
	public function find(sx:Int, sy:Int, tx:Int, ty:Int):Array<Int> {
		if (sx == tx && sy == ty) return new Array<Int>();
		if (!m[tx + ty * w].walk && !m[tx + ty * w].act) return [];
		for (t in m){
			t.from = null;
			t.cost = -1;
		}
		m[sx + sy * w].cost = 0;
		var cht = new Array<PFAstarTile>();
		addNeighbors(cht, m[sx + sy * w]);
		while (cht.length > 0){
			var ucht:PFAstarTile = null;
			if (cht.length == 1){
				ucht = cht.shift();
			} else if (cht.length == 2){
				if (heuristic(cht[1], tx, ty) < heuristic(cht[0], tx, ty)){
					ucht = cht.pop();
				} else {
					ucht = cht.shift();
				}
			} else {
				cht.sort(heuristicSort.bind(tx, ty, _, _));
				ucht = cht.shift();
			}
			if (ucht.x == tx && ucht.y == ty){
				var ret = new Array<Int>();
				var ct:PFAstarTile = ucht;
				var cx:Int = ucht.x;
				var cy:Int = ucht.y;
				while (cx != sx || cy != sy){
					ret.unshift(cy - ct.from.y);
					ret.unshift(cx - ct.from.x);
					ct = ct.from;
					cx = ct.x;
					cy = ct.y;
				}
				return ret;
			}
			addNeighbors(cht, ucht);
		}
		return [];
	}
	
	public function findAny(sx:Int, sy:Int, tgt:Array<Array<Int>>):Array<Int> {
		var shortest:Array<Int> = [];
		for (t in tgt){
			if (sx == t[0] && sy == t[1]) return [];
			var nadd = find(sx, sy, t[0], t[1]);
			if (nadd.length == 0) continue;
			if (shortest.length == 0 || nadd.length < shortest.length) shortest = nadd;
		}
		return shortest;
	}
	
	/*
	public function findAny(sx:Int, sy:Int, tgt:Array<Array<Int>>):Array<Int> {
		var retn:Bool = true;
		for (tt in tgt){
			if (sx == tt[0] && sy == tt[0]){
				return [];
			}
			if (m[tt[0] + tt[1] * w].walk || m[tt[0] + tt[1] * w].act){
				retn = false;
			}
		}
		if (retn) return [];
		for (t in m){
			t.from = null;
			t.cost = -1;
		}
		m[sx + sy * w].cost = 0;
		var cht = new Array<PFAstarTile>();
		addNeighbors(cht, m[sx + sy * w]);
		while (cht.length > 0){
			var ucht:PFAstarTile = null;
			if (cht.length == 1){
				ucht = cht.shift();
			} else {
				cht.sort(heuristicSortAny.bind(tgt, _, _));
				ucht = cht.shift();
			}
			for (tt in tgt){
				if (ucht.x == tt[0] && ucht.y == tt[1]){
					var ret = new Array<Int>();
					var ct:PFAstarTile = ucht;
					var cx:Int = ucht.x;
					var cy:Int = ucht.y;
					while (cx != sx || cy != sy){
						ret.unshift(cy - ct.from.y);
						ret.unshift(cx - ct.from.x);
						ct = ct.from;
						cx = ct.x;
						cy = ct.y;
					}
					return ret;
				}
			}
			addNeighbors(cht, ucht);
		}
		return [];
	}
	*/
	
	private inline function heuristic(t:PFAstarTile, tx:Int, ty:Int):Int {
		return FM.absI(tx - t.x) + FM.absI(ty - t.y);
	}
	
	private inline function heuristicSort(tx:Int, ty:Int, a:PFAstarTile, b:PFAstarTile):Int {
		return heuristic(a, tx, ty) - heuristic(b, tx, ty);
	}
	
	private inline function heuristicSortAny(tgt:Array<Array<Int>>, a:PFAstarTile, b:PFAstarTile):Int {
		var ahs:Array<Int> = tgt.map(function(tt:Array<Int>){return heuristic(a, tt[0], tt[1]);});
		var bhs:Array<Int> = tgt.map(function(tt:Array<Int>){return heuristic(b, tt[0], tt[1]);});
		var mina:Int = ahs[0];
		var minb:Int = bhs[0];
		for (i in 1...ahs.length){
			mina = (ahs[i] < mina ? ahs[i] : mina);
			minb = (bhs[i] < minb ? bhs[i] : minb);
		}
		return mina - minb;
	}
	
	private inline function addNeighbors(a:Array<PFAstarTile>, t:PFAstarTile):Void {
		for (x in -1...2){
			for (y in -1...2){
				if (x == 0 && y == 0) continue;
				if (t.x + x < 0 || t.x + x >= w || t.y + y < 0 || t.y + y >= h) continue;
				var ni:Int = t.x + x + (t.y + y) * w;
				if (!m[ni].walk && !m[ni].act) continue;
				if (m[ni].cost == -1){
					m[ni].from = t;
					m[ni].cost = t.cost + 1;
					a.push(m[ni]);
				}/* else (m[ni].cost > t.cost + 1){
					
				}*/
			}
		}
	}
}

class PFAstarTile {
	public var walk:Bool;
	public var act:Bool;
	public var x:Int;
	public var y:Int;
	public var from:PFAstarTile;
	public var cost:Int;
	
	public function new(nwalk:Bool, nact:Bool, nx:Int, ny:Int){
		walk = nwalk;
		act = nact;
		x = nx;
		y = ny;
	}
}