package sk.thenet.plustd.audio.music;

import haxe.ds.Vector;
import sk.thenet.plustd.FM;

class MMPatternClone extends MMPattern {
	public static function fromNotes(pat:String, ntempo:Float, norig:MMPatternBasic):MMPatternClone {
		var notes:Array<String> = pat.split(" ");
		var noff:Array<Float> = [];
		var ntrig:Array<Int> = [];
		var effects:Array<MMEffect> = [];
		var effects_step:Array<Int> = [];
		var lnote:Float = -1;
		for (nf in 0...notes.length){
			var nfe = notes[nf].split(";");
			var n = nfe[0];
			if (n == "---"){
				noff.push(lnote);
				ntrig.push(0);
			} else if (n == "STP"){
				noff.push(-1);
				ntrig.push(-1);
			} else if (n == "TRG"){
				noff.push(-1);
				ntrig.push(1);
			} else {
				lnote = Std.parseFloat(n);
				noff.push(Std.parseFloat(n));
				ntrig.push(1);
			}
			for (ni in 1...nfe.length){
				var par:Array<String> = nfe[ni].substr(3).split(",");
				switch (nfe[ni].substr(0, 3)){
					case "sli":
					effects.push(new MMESlide(false, Std.parseInt(par[0]),
						MMPattern.countSamples(Std.parseFloat(par[1]), ntempo)));
					case "arp":
					effects.push(new MMEArpeggio(Std.parseInt(par[0]), Std.parseInt(par[1]), Std.parseInt(par[2]),
						MMPattern.countSamples(Std.parseFloat(par[3]), ntempo)));
					case "slj":
					effects.push(new MMESlide(true, Std.parseInt(par[0]),
						MMPattern.countSamples(Std.parseFloat(par[1]), ntempo)));
					case "VOL":
					effects.push(new MMEFade(true, MMPattern.countSamples(Std.parseFloat(par[0]), ntempo)));
					case "vol":
					effects.push(new MMEFade(false, MMPattern.countSamples(Std.parseFloat(par[0]), ntempo)));
					default: continue;
				}
				effects_step.push(nf);
			}
		}
		var ret = new MMPatternClone(noff, ntrig, ntempo, norig);
		ret.effects = effects;
		ret.effects_step = effects_step;
		return ret;
	}
	
	public var off:Array<Float>;
	public var noff:Array<Float>;
	public var trig:Array<Int>;
	public var orig:MMPatternBasic;
	public var fph:Int = 0;
	public var tph:Float = 0;
	public var fph_trig:Bool = false;
	
	public function new(noff:Array<Float>, ntrig:Array<Int>, ntempo:Float, norig:MMPatternBasic){
		super(ntempo);
		off = noff;
		trig = ntrig;
		orig = norig;
	}
	
	override public function reset():Void {
		super.reset();
		fph = 0;
		tph = 0;
		fph_trig = false;
	}
	
	override public inline function setFreq(instr:MMInstrument, c:Int):Void {
		if (instr != null){
			/*if (off[fph] == 0){
				instr.notes = orig.notes[orig.fph];
				instr.freq = orig.freq[orig.fph];
			} else {*/
			instr.notes = orig.notes[orig.fph] + off[fph];
			instr.freq = MusicManager.indexToFreq(instr.notes);
			//}
			instr.trig = (fph_trig ? 0 : trig[fph]);
		}
		fph_trig = true;
		if (!eftrig && effects.length > 0 && effects_step[efph] == fph){
			var lg:Int = 0;
			while (lg++ < effects.length && effects_step[efph] == fph){
				if (instr != null) instr.addPatternEffect(effects[efph], c);
				effects[efph].trigger(1, MMEffect.ET_PATTERN);
				efph++;
				efph %= effects.length;
			}
			eftrig = true;
		}
		tph += 1;
		if (tph >= tempo){
			fph++;
			fph_trig = false;
			eftrig = false;
			while (fph >= off.length){
				for (e in effects) e.reset();
				fph -= off.length;
			}
			tph -= tempo;
		}
	}
}