package sk.thenet.plustd.audio.music;

import haxe.ds.Vector;
import haxe.ds.StringMap;
import flash.media.Sound;
import flash.media.SoundChannel;
import flash.events.SampleDataEvent;
import flash.utils.ByteArray;

typedef MMAutomation = MusicManager->Int->Void;

class MusicManager {
	public static var PATTERNS:Array<Array<Int>> = [
		[0],
		[0, 4, 7],
		[0, 3, 7],
		[0, 4],
		[0, 3],
		[0, 2, 3, 4, 7, 9],
		[0, 3, 5, 6, 7, 10],
		[0, 2, 3, 5, 6, 9, 10],
		[0, 2, 4, 5, 7, 9, 11],
		[0, 2, 3, 5, 7, 8, 10],
		[0, 2, 3, 5, 7, 8, 11],
		[0, 2, 3, 5, 7, 9, 10]
	];
	public static inline var PATTERN_OCTAVE:Int = 0;
	public static inline var PATTERN_TRIAD_MAJOR:Int = 1;
	public static inline var PATTERN_TRIAD_MINOR:Int = 2;
	public static inline var PATTERN_FAMICHORD_MAJOR:Int = 3;
	public static inline var PATTERN_FAMICHORD_MINOR:Int = 4;
	public static inline var PATTERN_BLUES_MAJOR:Int = 5;
	public static inline var PATTERN_BLUES_MINOR:Int = 6;
	public static inline var PATTERN_BLUES_HEPTA:Int = 7;
	public static inline var PATTERN_MAJOR:Int = 8;
	public static inline var PATTERN_MINOR_NATURAL:Int = 9;
	public static inline var PATTERN_MINOR_HARMONIC:Int = 10;
	public static inline var PATTERN_DORIC:Int = 11;
	
	public static inline var SAMPLERATE:Int = 44100;
	
	public static var NOTES:Array<String> = [
		"C-1",
		"C#1",
		"D-1",
		"D#1",
		"E-1",
		"F-1",
		"F#1",
		"G-1",
		"G#1",
		"A-1",
		"A#1",
		"H-1",
		"C-2",
		"C#2",
		"D-2",
		"D#2",
		"E-2",
		"F-2",
		"F#2",
		"G-2",
		"G#2",
		"A-2",
		"A#2",
		"H-2",
		"C-3",
		"C#3",
		"D-3",
		"D#3",
		"E-3",
		"F-3",
		"F#3",
		"G-3",
		"G#3",
		"A-3",
		"A#3",
		"H-3",
		"C-4",
		"C#4",
		"D-4",
		"D#4",
		"E-4",
		"F-4",
		"F#4",
		"G-4",
		"G#4",
		"A-4", // mid, index 45
		"A#4",
		"H-4",
		"C-5",
		"C#5",
		"D-5",
		"D#5",
		"E-5",
		"F-5",
		"F#5",
		"G-5",
		"G#5",
		"A-5",
		"A#5",
		"H-5",
		"C-6",
		"C#6",
		"D-6",
		"D#6",
		"E-6",
		"F-6",
		"F#6",
		"G-6",
		"G#6",
		"A-6",
		"A#6",
		"H-6",
		"C-7",
		"C#7",
		"D-7",
		"D#7",
		"E-7",
		"F-7",
		"F#7",
		"G-7",
		"G#7",
		"A-7",
		"A#7",
		"H-7"
	];
	public static inline function noteToIndex(note:String, acc:Float):Float {
		var nin:Int = NOTES.indexOf(note);
		return (nin < 0 ? 45 : nin);
	}
	public static inline function noteToFreq(note:String, acc:Float):Float {
		// TO WAVELEN!!
		var nin:Int = NOTES.indexOf(note);
		return indexToFreq(nin < 0 ? 45 : nin);
	}
	public static inline function indexToFreq(index:Float):Float {
		// TO WAVELEN!!
		return SAMPLERATE / (440 * Math.pow(2, (index - 45) / 12));
	}
	/*
	public static inline function wavelenToIndex(wavelen:Float):Float {
		return 12 * (Math.log((SAMPLERATE / n) / 440) / ln2) + 45;
	}
	public static inline function freqToIndex(freq:Float):Float {
		return 12 * (Math.log(n / 440) / ln2) + 45;
	}
	*/
	
	public var instruments:Array<MMInstrument>;
	public var patterns:Array<MMPattern>;
	public var auto:Vector<MMAutomation>;
	//public var snd:Sound;
	public var sch:SoundChannel;
	
	public var sections:StringMap<Array<MMPattern>>;
	public var sectauto:StringMap<Vector<MMAutomation>>;
	public var sectlen:StringMap<Int>;
	public var sectprog:Array<String>;
	public var sectprogpos:Int;
	public var sectpos:Int;
	
	public var rendering:Bool;
	public var rendered:Int;
	public var rendermul:Float;
	public var playing:Bool;
	public var loop:Bool;
	
	public var render_max:Float;
	public var render_min:Float;
	
	public function new(ninstr:Array<MMInstrument> = null, neff:Array<Array<MMEffect>> = null){
		patterns = [];
		sections = new StringMap<Array<MMPattern>>();
		sectauto = new StringMap<Vector<MMAutomation>>();
		sectlen = new StringMap<Int>();
		sectprog = [];
		sectprogpos = 0;
		sectpos = 0;
		rendering = false;
		playing = false;
		loop = true;
		instruments = [];
		if (ninstr != null){
			for (i in 0...ninstr.length){
				addInstrument(ninstr[i], (neff != null ? (i < neff.length ? neff[i] : []) : []));
			}
		}
	}
	
	public function addSection(name:String, len:Int, nsect:Array<MMPattern>, nauto:Array<MMAutomation>):Void {
		sections.set(name, nsect);
		sectauto.set(name, new Vector<MMAutomation>(nauto.length));
		for (ai in 0...nauto.length){
			sectauto.get(name)[ai] = nauto[ai];
		}
		if (len == -1) len = 0;
		else if (len == 0) len = FM.floor(cast(nsect[0], MMPatternBasic).notes.length * nsect[0].tempo);
		sectlen.set(name, len);
	}
	
	public function addInstrument(instr:MMInstrument, neff:Array<MMEffect>):Void {
		var addi:Int = instruments.length;
		instruments[addi] = instr;
		for (i in 0...neff.length){
			instruments[addi].addEffect(neff[i]);
		}
	}
	
	public function setPattern(iid:Int, pat:MMPattern):Void {
		patterns[iid] = pat;
	}
	
	private function setProgression(nsectprog:Array<String>):Void {
		sectprog = nsectprog;
		sectprogpos = 0;
		sectpos = 0;
		patterns = [];
		var npat = sections.get(sectprog[sectprogpos]);
		for (i in 0...npat.length){
			patterns[i] = npat[i];
			patterns[i].reset();
		}
		auto = sectauto.get(sectprog[sectprogpos]);
	}
	
	public function playProgression(nsectprog:Array<String>, nloop:Bool = true):Void {
		if (playing) return;
		setProgression(nsectprog);
		playing = true;
		loop = nloop;
		var snd = new Sound();
		snd.addEventListener(SampleDataEvent.SAMPLE_DATA, playSample);
		sch = snd.play();
	}
	
	public function stop():Void {
		if (playing){
			sch.stop();
			sch = null;
			playing = false;
		}
	}
	
	private function playSample(e:SampleDataEvent):Void {
		sample(e.data, true);
	}
	
	public function saveProgression(nsectprog:Array<String>, savemul:Float = 1):Void {
		setProgression(nsectprog);
		
		var wavdata = new ByteArray();
		wavdata.endian = LITTLE_ENDIAN;
		rendering = true;
		rendered = 0;
		render_min = Math.POSITIVE_INFINITY;
		render_max = Math.NEGATIVE_INFINITY;
		rendermul = savemul;
		while (rendering){
			sample(wavdata, false);
		}
		
		var wav = new ByteArray();
		var bitDepth = 16;
		var channels = 2;
		var blockAlign:Int = bitDepth >> 3;
		var soundLength:UInt = rendered * blockAlign * channels;
		var filesize:Int = 36 + soundLength;
		var bytesPerSec:Int = 44100 * blockAlign * channels;
		var wav:ByteArray = new ByteArray();
		// Header
		wav.endian = BIG_ENDIAN;
		wav.writeUnsignedInt(0x52494646);		// Chunk ID "RIFF"
		wav.endian = LITTLE_ENDIAN;
		wav.writeUnsignedInt(filesize);			// Chunck Data Size
		wav.endian = BIG_ENDIAN;
		wav.writeUnsignedInt(0x57415645);		// RIFF Type "WAVE"
		// Format Chunk
		wav.endian = BIG_ENDIAN;
		wav.writeUnsignedInt(0x666D7420);		// Chunk ID "fmt "
		wav.endian = LITTLE_ENDIAN;
		wav.writeUnsignedInt(16);				// Chunk Data Size
		wav.writeShort(1);						// Compression Code PCM
		wav.writeShort(channels);				// Number of channels
		wav.writeUnsignedInt(44100);			// Sample rate
		wav.writeUnsignedInt(bytesPerSec);		// Average bytes per second
		wav.writeShort(blockAlign);				// Block align
		wav.writeShort(bitDepth);				// Significant bits per sample
		// Data Chunk
		wav.endian = BIG_ENDIAN;
		wav.writeUnsignedInt(0x64617461);		// Chunk ID "data"
		wav.endian = LITTLE_ENDIAN;
		wav.writeUnsignedInt(soundLength);		// Chunk Data Size
		wav.writeBytes(wavdata, wav.position);
		
		var fr = new flash.net.FileReference();
		fr.save(wav);
		
		//synthWave(wav, _envelopeFullLength, false, sampleRate, bitDepth);
		//wav.position = 0;
		//return wav;
	}
	
	public function sample(data:ByteArray, play:Bool):Void {
		for (c in 0...8192){
			if (play ? !playing : !rendering) return;
			for (a in auto){
				a(this, sectpos);
			}
			var outl:Float = 0;
			var outr:Float = 0;
			for (i in 0...patterns.length){
				patterns[i].setFreq(i >= instruments.length ? null : instruments[i], c);
			}
			for (instr in instruments){
				if (instr.effects_pos.length > 0 && c == instr.effects_pos[0]){
					instr.effects.push(instr.effects_buf.shift());
					instr.effects_pos.shift();
				}
				var oe = instr.effects;
				instr.effects = [];
				for (e in oe){
					if (e.done) continue;
					instr.effects.push(e);
					if (e.tapl != MMEffect.ETA_NOTES) continue;
					e.applyNotes(instr, c);
				}
				if (instr.trig > 0){
					instr.gate = 1;
					if (instr.trig == 1){
						for (e in instr.effects){
							e.trigger(instr.trig, MMEffect.ET_INSTRUMENT);
						}
					}
					if (instr.lfreq != instr.freq && instr.lfreq > 0)
						instr.ph *= instr.freq / instr.lfreq;
					instr.lfreq = instr.freq;
				} else if (instr.trig < 0){
					var chg:Bool = true;
					for (e in instr.effects){
						if (e.gate_always) chg = false;
						e.trigger(instr.trig, MMEffect.ET_INSTRUMENT);
					}
					if (chg) instr.gate = 0;
				}
				var buf:Float = instr.buffer();
				for (e in instr.effects){
					e.clock();
					if (e.tapl != MMEffect.ETA_BUFFER) continue;
					buf = e.apply(buf);
				}
				buf *= .4 * instr.volume;
				outl += buf * (instr.pan <= .5 ? 1 : 1 - (instr.pan - .5) / .5);
				outr += buf * (instr.pan >= .5 ? 1 : instr.pan / .5);
				instr.ph += 1;
				if (instr.lfreq > 0){
					while (instr.ph >= instr.lfreq) instr.ph -= instr.lfreq;
				}
			}
			if (play){
				data.writeFloat(outl);
				data.writeFloat(outr);
			} else {
				data.writeShort(FM.floor(outl * 32760 * rendermul));
				data.writeShort(FM.floor(outr * 32760 * rendermul));
				rendered++;
			}
			sectpos++;
			if (sectpos >= sectlen.get(sectprog[sectprogpos]) && sectlen.get(sectprog[sectprogpos]) != 0){
				sectprogpos++;
				if (sectprogpos >= sectprog.length){
					if (!play){
						rendering = false;
					} else if (!loop && play){
						playing = false;
					}
					sectprogpos = 0;
				}
				patterns = [];
				var npat = sections.get(sectprog[sectprogpos]);
				for (i in 0...npat.length){
					patterns[i] = npat[i];
					patterns[i].reset();
				}
				sectpos = 0;
			}
		}
	}
}